from django.conf import settings

# The cache config from django.core.cache.caches to use for backends.cache.CacheBackend
# default to django.core.cache.caches['default']
MAILVIEWER_CACHE = getattr(settings, 'MAILVIEWER_CACHE', 'default')
MAILVIEWER_DATABASE_BACKEND_MODEL = getattr(settings, 'MAILVIEWER_DATABASE_BACKEND_MODEL', 'mail_viewer_database_backend.EmailMessage')



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class DatabaseBackendConfig(AppConfig):
    label = 'mail_viewer_database_backend'
    name = 'django_mail_viewer.backends.database'
    # May want to change this verbose_name
    verbose_name = _("Database Backend")



from django import template
from django.utils.safestring import mark_safe

register = template.Library()


@register.simple_tag
def message_attribute(message, attribute):
    """
    Return the attribute from the SafeMIMEMessage

    This is required to deal with case insensitivity and some attributes use
    a hyphen in the name.  In a template, message.message-id, for example does
    not work due to the hyphen.
    """
    return message.get(attribute)


@register.simple_tag
def message_lookup_id(message):
    """
    Return the message id of an email message.

    Useful because the value is stored in a dict with a hyphen in the key,
    making it inaccessible directly.
    """

    return mark_safe(message.get('message-id', '').strip('<>'))


@register.simple_tag
def display_message_attribute(message, attribute):
    return mark_safe(message_attribute(message, attribute))



from django.urls import re_path

from . import views

urlpatterns = [
    re_path(
        r'message/(?P<message_id>.+)/attachment/(?P<attachment>.+)/$',
        views.EmailAttachmentDownloadView.as_view(),
        name="mail_viewer_attachment",
    ),
    re_path(r'message/(?P<message_id>.+)/delete/$', views.EmailDeleteView.as_view(), name="mail_viewer_delete"),
    re_path(r'message/(?P<message_id>.+)/$', views.EmailDetailView.as_view(), name='mail_viewer_detail'),
    re_path(r'', views.EmailListView.as_view(), name='mail_viewer_list'),
]



import email.message
import email.utils
import json
from typing import TYPE_CHECKING, Any, Dict, List, Union

from django.db import models


class AbstractBaseEmailMessage(models.Model):
    """
    Abstract base class for email messages allowing users to easily make their own class with a custom
    storage for the Message FileField. Once Django 3.1+ only is supported then a callable may be used
    rendering this not really necessary.

    To customize, subclass this and add a `FileField()` named `serialized_message_file` with the storage you would
    like to use.
    """

    # Technically optional, but really should be there according to RFC 5322 section 3.6.4
    # and Django always creates the message_id on the main part of the message so we know
    # it will be there, but not for all sub-parts of a multi-part message
    message_id = models.CharField(max_length=250, blank=True, default='')
    # Would love to make message_headers be a JSONField, but do not want to tie this to
    # postgres only.
    message_headers = models.TextField()
    content = models.TextField(blank=True, default='')
    parent = models.ForeignKey(
        'self', blank=True, null=True, default=None, related_name='parts', on_delete=models.CASCADE
    )
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    class Meta:
        abstract = True

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        # methods here expect the concrete subclasses to implement the file_attachment field
        # should only be necessary until django 2.2 support is dropped... I hope
        if TYPE_CHECKING and not hasattr(self, 'file_attachment'):
            self.file_attachment = models.FileField(blank=True, default='', upload_to='mailviewer_attachments')

    # I really only need/use get_filename(), get_content_type(), get_payload(), walk()
    # returns Any due to failobj
    def get(self, attr: str, failobj: Any = None) -> Any:
        """
        Get a header value.

        Like __getitem__() but return failobj instead of None when the field
        is missing.
        """
        # Or should I muck with __getitem__ and __setitem__, etc
        # like in https://github.com/python/cpython/blob/3.8/Lib/email/message.py#L382
        vals = {k.lower(): v for k, v in self.headers().items()}
        if vals:
            lower_attr = attr.lower()
            return vals.get(lower_attr, failobj)
        return failobj

    def date(self) -> str:
        return self.get('date')

    def is_multipart(self) -> bool:
        """
        Returns True if the message is multipart
        """
        # Not certain the self.parts.all() is accurate
        return self.get_content_type() == 'rfc/822' or self.parts.exists()  # type: ignore

    def headers(self) -> Dict[str, str]:
        """
        Return the Messages email headers as a dict
        """
        return json.loads(self.message_headers)

    def values(self) -> Dict[str, str]:
        """
        Return the Messages email headers as a dict
        """
        # not sure this is right...
        return self.headers()

    def walk(self) -> 'Union[models.QuerySet[AbstractBaseEmailMessage], List[AbstractBaseEmailMessage]]':
        if not self.parts.all().exists():  # type: ignore
            # Or should I be saving a main message all the time and even just a plaintext has a child part, hmm
            return [self]
        return self.parts.all().order_by('-created_at', 'id')  # type: ignore

    def get_param(self, param: str, failobj=None, header: str = 'content-type', unquote: bool = True) -> str:
        """
        Return the value of the Content-Type header’s parameter param as a string. If the message has no Content-Type header or if there is no such parameter, then failobj is returned (defaults to None).

        Optional header if given, specifies the message header to use instead of Content-Type.
        See https://docs.python.org/3/library/email.compat32-message.html#email.message.Message.get_param
        """
        # TODO: error handling skipped for sure here... need to see what the real email message does
        # Should also consider using cgi.parse_header
        h = self.get(header)
        params = h.split(';')
        for part in params[1:]:
            part_name, part_val = part.split('=')
            part_name = part_name.strip()
            part_val = part_val.strip()
            if part_name == param:
                return part_val
        return ''

    def get_payload(
        self, i: Union[int, None] = None, decode: bool = False
    ) -> 'Union[bytes, AbstractBaseEmailMessage, models.QuerySet[AbstractBaseEmailMessage]]':
        """
        Temporary backwards compatibility with email.message.Message
        """
        # TODO: sort out type hint for return value here. Maybe use monkeytype to figure this out.
        if not self.is_multipart():
            charset = self.get_param('charset')
            if self.file_attachment:
                self.file_attachment.seek(0)
                try:
                    return self.file_attachment.read()
                finally:
                    self.file_attachment.seek(0)
            else:
                # our content is a str but get_payload() returns bytes normally so we need to re-encode it... yeah.
                return self.content.encode(charset)

        parts = self.parts.all()  # type: ignore
        if i:
            return parts[i]
        return parts

    def get_content_type(self) -> str:
        """
        Return's the message's content-type or mime type.
        """
        h = self.get('content-type')
        params = h.split(';')
        return params[0]

    def get_filename(self, failobj=None) -> str:
        content_disposition = self.headers().get('Content-Disposition', '')
        parts = content_disposition.split(';')
        for part in parts:
            if part.strip().startswith('filename'):
                filename = part.split('=')[1].strip('"').strip()
                return email.utils.unquote(filename)
        return ''


class EmailMessage(AbstractBaseEmailMessage):
    """
    Django model mimicking the minimal implementation of email.message.Message needed for django-mail-viewer.

    An Email Message where the base message is stored in the database and attachments are stored in a `mailviewer_attachments/
    directory in your default media storage.

    To customize storage, subclass AbstractBaseEmailMessage and add a `FileField()` named `file_attachment`
    with the storage you would like to use. This may be because the default media storage is public readable or
    it just needs to be stored elsewhere, such as locally, or a different s3 bucket than the default storage.
    """

    file_attachment = models.FileField(blank=True, default='', upload_to='mailviewer_attachments')

    class Meta:
        db_table = 'mail_viewer_emailmessage'
        ordering = ('id',)
        indexes = [models.Index(fields=['message_id'])]



"""
Backend for test environment.
"""
from django.core import mail
from django.core.mail.backends.base import BaseEmailBackend


class EmailBackend(BaseEmailBackend):
    """
    An email backend to use during testing and local development with Django Mail Viewer.

    Similar to django.core.backends.locmem.EmailBackend, this adds an outbox attribute ot
    django.core.mail.  This stores the EmailMessage object as well as message.message().
    This is because many of the headers are generated at the time message.message() is called
    and are not stored by the default locmem backend, including ones useful for consistently
    looking up a specific message even if the list is reordered.
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        if not hasattr(mail, 'outbox'):
            mail.outbox = []

    def send_messages(self, messages):
        msg_count = 0
        for message in messages:
            m = message.message()
            mail.outbox.append(m)
            msg_count += 1
        return msg_count

    def get_message(self, lookup_id):
        """
        Look up and return a specific message in the outbox
        """
        for message in mail.outbox:
            # if a user is manually passing in Message-ID in extra_headers and capitalizing it
            # differently than the expected Message-ID,  which is suppored by
            # EmailMessage.message(), then we can't just access the key directly.  Instead iterate
            # over the keys and vls
            if message.get('message-id') == lookup_id:
                return message
        return None

    def get_outbox(self, *args, **kwargs):
        """
        Get the outbox used by this backend.  This backend returns a copy of mail.outbox.
        May add pagination args/kwargs.
        """
        return getattr(mail, 'outbox', [])[:]

    def delete_message(self, message_id: str):
        """
        Remove the message with the given id from the mailbox
        """
        outbox = getattr(mail, 'outbox', [])
        index_to_remove = None
        for idx, message in enumerate(outbox):
            if message.get('message-id') == message_id:
                index_to_remove = idx
                break
        if index_to_remove is not None:
            del outbox[index_to_remove]



"""
Backend for test environment.
"""
from django.core import cache
from django.core.mail.backends.base import BaseEmailBackend

from .. import settings as mailviewer_settings


class EmailBackend(BaseEmailBackend):
    """
    An email backend to use during testing and local development with Django Mail Viewer.

    Uses Django's cache framework to store sent emails in the cache so that they can
    be easily retrieved in multi-process environments such as using Django Channels or
    when sending an email from a python shell.
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.cache = cache.caches[mailviewer_settings.MAILVIEWER_CACHE]
        # a cache entry with a list of the rest of the cache keys
        # This is for get_outbox() so that the system knows which cache keys are there
        # to retrieve them. Django does not have a built in way to get the keys
        # which exist in the cache.
        self.cache_keys_key = 'message_keys'

    def send_messages(self, messages):
        msg_count = 0
        for message in messages:
            m = message.message()
            message_id = m.get('message-id')
            self.cache.set(message_id, m)

            # if multiple processes are updating this at the same time then
            # things will get hung up.  May introduce a lock key and spinlock
            # to avoid clobbering the value stored in the list of keys.
            current_cache_keys = self.cache.get(self.cache_keys_key)
            if not current_cache_keys:
                current_cache_keys = []
            current_cache_keys.append(message_id)
            self.cache.set(self.cache_keys_key, current_cache_keys)
            msg_count += 1
        return msg_count

    def get_message(self, lookup_id):
        """
        Look up and return a specific message in the outbox
        """
        return self.cache.get(lookup_id)

    def get_outbox(self, *args, **kwargs):
        """
        Get the outbox used by this backend.  This backend returns a copy of mail.outbox.
        May add pagination args/kwargs.
        """
        # grabs all of the keys in the stored self.cache_keys_key
        # and passes those into get_many() to retrieve the keys
        message_keys = self.cache.get(self.cache_keys_key)
        if message_keys:
            messages = list(self.cache.get_many(message_keys).values())
        else:
            messages = []
        return messages

    def delete_message(self, message_id: str):
        """
        Remove the message with the given id from the mailbox
        """
        message_keys = self.cache.get(self.cache_keys_key, [])
        message_keys.remove(message_id)
        self.cache.set(self.cache_keys_key, message_keys)
        self.cache.delete(message_id)



from io import BytesIO

from django.core import mail
from django.http import Http404, HttpResponse, HttpResponseRedirect
from django.urls import reverse
from django.utils.encoding import smart_str
from django.views.generic.base import TemplateView, View


class SingleEmailMixin:
    """
    Mixin for details for a single email
    """

    def get_message(self):
        message = None
        with mail.get_connection() as connection:
            message_id = self.kwargs.get('message_id')
            # TODO: put this fiddling with brackets on the backend itself...
            message = connection.get_message(f'<{message_id}>')
        return message

    def _parse_email_attachment(self, message, decode_file=True):
        """
        Parse an attachment out of an email.message.Message object.

        params:
        msg: email.message.Message object
        decode_file: Boolean whether to decode the base64 encoded file to an actual file or not
        """
        # TODO: move this all to the email backend. It exists on the db backend already
        # but maybe it could live on BaseEmailBackend - or just in a function not in a class
        # because right now the database email backend is the only one which needs it
        content_disposition = message.get("Content-Disposition", None)
        if content_disposition:
            dispositions = content_disposition.strip().split(";")
            if bool(content_disposition and dispositions[0].lower() == "attachment"):
                if decode_file:
                    file_data = message.get_payload(decode=True)
                    attachment = BytesIO(file_data)
                    attachment.size = len(file_data)

                    attachment.content_type = message.get_content_type()
                    attachment.name = None
                    attachment.create_date = None
                    attachment.mod_date = None
                    attachment.read_date = None
                else:
                    attachment = None
                for param in dispositions[1:]:
                    name, value = param.split("=")
                    name = name.lower()

                    # Since I am terrible and left terrible comments I am assuming the
                    # datetime TODO comments below mean to store as a datetime.datetime object
                    if name == "filename":
                        attachment.name = value
                    elif name == "create-date":
                        attachment.create_date = value  # TODO: datetime
                    elif name == "modification-date":
                        attachment.mod_date = value  # TODO: datetime
                    elif name == "read-date":
                        attachment.read_date = value  # TODO: datetime
                return {
                    # 'filename': Path(message.get_filename()).name,  ??
                    'filename': message.get_filename(),
                    'content_type': message.get_content_type(),
                    'file': attachment,
                }
        return None

    def _parse_email_parts(self, message, decode_files=True):
        # much of this borrowed from
        # https://www.ianlewis.org/en/parsing-email-attachments-python
        attachments = []
        body = None
        html = None
        for part in message.walk():
            attachment = self._parse_email_attachment(part, decode_files)
            if attachment:
                attachments.append(attachment)
            elif part.get_content_type() == 'text/plain':
                # should we get the default charset from the system if no charset?
                # decode=True handles quoted printable and base64 encoded data
                charset = part.get_param('charset')
                body = part.get_payload(decode=True).decode(charset, errors='replace')
            elif part.get_content_type() == 'text/html':
                # original code set html to '' if it was None and then appended
                # as if we might have multiple html parts which are just one html message?
                charset = part.get_param('charset')
                html = part.get_payload(decode=True).decode(charset, errors='replace')

        subject = message.get('subject')
        msg_from = message.get('from')
        to = message.get('to')
        return (subject, body, html, msg_from, to, attachments)


class EmailListView(TemplateView):
    """
    Display a list of sent emails.
    """

    template_name = 'mail_viewer/email_list.html'

    def get_context_data(self, **kwargs):
        # TODO: need to make a custom backend which sets a predictable message-id header.
        # built in locmem uses a random number each time the message is accessed
        # preventing lookup in the detail view
        with mail.get_connection() as connection:
            # add a backend.get_outbox() for supporting multiple backends?
            outbox = connection.get_outbox()
        return super().get_context_data(outbox=outbox, **kwargs)


class EmailDetailView(SingleEmailMixin, TemplateView):
    """
    Display details of an email
    """

    template_name = 'mail_viewer/email_detail.html'

    def get_template_names(self):
        if self.request.headers.get('hx-request'):
            return ['mail_viewer/email_detail_content_fragment.html']
        return super().get_template_names()

    def get(self, request, *args, **kwargs):
        self.message = self.get_message()
        if not self.message:
            # Instead of default self.get() behavior and letting get_message() raise 404
            # because I want to stay within mailviewer and not dump out to a system's 404 page.
            return HttpResponseRedirect(reverse('mail_viewer_list'))

        response = super().get(request, *args, **kwargs)
        response['HX-Trigger-After-Swap'] = 'htmxEmailLoaded'
        return response

    def get_context_data(self, **kwargs):
        lookup_id = kwargs.get('message_id')
        message = self.message

        with mail.get_connection() as connection:
            outbox = connection.get_outbox()

        subject, text_body, html_body, sender, to, attachments = self._parse_email_parts(message, decode_files=False)
        return super().get_context_data(
            lookup_id=lookup_id,
            message=message,
            text_body=text_body,
            html_body=html_body,
            subject=subject,
            sender=sender,
            to=to,
            attachments=attachments,
            outbox=outbox,
            **kwargs,
        )


class EmailAttachmentDownloadView(SingleEmailMixin, View):
    """
    Stream out an email attachment to the web browser
    """

    def get_attachment(self, message):
        # TODO: eventually will need to handle different ways of having these
        # atachments stored.  Probably handle that on the EmailBackend class
        requested = int(self.kwargs.get('attachment'))
        i = 0
        # TODO: de-nest this some
        # TODO: use enumerate()...
        for part in message.walk():
            content_disposition = part.get("Content-Disposition", '')
            dispositions = content_disposition.strip().split(";")
            if content_disposition and dispositions[0].lower() == "attachment":
                if i == requested:
                    return self._parse_email_attachment(part, True)
                i += 1
        raise Http404('Attachment not found')

    def get(self, request, *args, **kwargs):
        message = self.get_message()
        attachment = self.get_attachment(message)
        response = HttpResponse(attachment['file'], content_type=attachment['content_type'])
        response['Content-Disposition'] = 'attachment; filename=%s' % smart_str(attachment['filename'])
        return response


class EmailDeleteView(SingleEmailMixin, TemplateView):
    """
    Delete an email. Works like Django's DeleteView but its not tied
    to a model.
    """

    template_name = 'mail_viewer/email_delete.html'

    def get_context_data(self, **kwargs):
        lookup_id = kwargs.get('message_id')
        message = self.message

        with mail.get_connection() as connection:
            outbox = connection.get_outbox()

        subject, text_body, html_body, sender, to, attachments = self._parse_email_parts(message, decode_files=False)
        return super().get_context_data(
            lookup_id=lookup_id,
            message=message,
            text_body=text_body,
            html_body=html_body,
            subject=subject,
            sender=sender,
            to=to,
            attachments=attachments,
            outbox=outbox,
            **kwargs,
        )

    def get(self, request, *args, **kwargs):
        self.message = self.get_message()
        if not self.message:
            # Instead of default self.get() behavior and letting get_message() raise 404
            # because I want to stay within mailviewer and not dump out to a system's 404 page.
            return HttpResponseRedirect(reverse('mail_viewer_list'))

        return super().get(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        """
        Delete the message from the outbox
        """
        # TODO: Should this be on its own view and support GET requests as well just to function with minimal javascript in the browser?
        message_id = self.kwargs.get('message_id')
        with mail.get_connection() as connection:
            pass
            # TODO: put this fiddling with brackets on the backend itself...
            # cache and database backends would function without brackets, although they would need to remove them
            # from the original data.
            connection.delete_message(f'<{message_id}>')

        # apparently htmx POST requests do not send as XmlHttpRequest?
        is_ajax = request.headers.get('x-requested-with') == 'XMLHttpRequest'
        if is_ajax or request.headers.get('hx-request'):
            response = HttpResponse('', status=200)
            current_url = request.META.get('HTTP_HX_CURRENT_URL', '')
            if message_id in current_url:
                response['HX-Redirect'] = reverse('mail_viewer_list')
        else:
            response = HttpResponseRedirect(reverse('mail_viewer_list'))
        return response



from django.contrib import admin

from .models import EmailMessage


class EmailMessageAdmin(admin.ModelAdmin):
    list_display = ('pk', 'parent', 'message_id', 'created_at', 'updated_at')
    search_fields = ('pk', 'message_id', 'message_headers')
    readonly_fields = ('created_at', 'updated_at')


admin.site.register(EmailMessage, EmailMessageAdmin)



#!/usr/bin/env python
import os
import sys

from setuptools import find_packages, setup

version = '2.1.0'

if sys.argv[-1] == 'publish':
    try:
        import wheel

        print("Wheel version: ", wheel.__version__)
    except ImportError:
        print('Wheel library missing. Please run "pip install wheel"')
        sys.exit()
    os.system('python setup.py sdist upload')
    os.system('python setup.py bdist_wheel upload')
    sys.exit()

if sys.argv[-1] == 'tag':
    print("Tagging the version on git:")
    os.system("git tag -a %s -m 'version %s'" % (version, version))
    os.system("git push --tags")
    sys.exit()

readme = open('README.rst').read()

setup(
    name='django-mail-viewer',
    version=version,
    description="""View emails in development without actually sending them.""",
    long_description=readme,
    author='Justin Michalicek',
    author_email='jmichalicek@gmail.com',
    url='https://github.com/jmichalicek/django-mail-viewer',
    packages=find_packages(exclude=['tests', 'docs']),
    include_package_data=True,
    install_requires=[],
    license="MIT",
    zip_safe=False,
    keywords=['django-mail-viewer', 'django', 'email',],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Framework :: Django',
        'Framework :: Django :: 3.2',
        'Framework :: Django :: 4.0',
        'Framework :: Django :: 4.1',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
    ],
)



import json
from io import BytesIO
from pathlib import Path

from django.apps import apps
from django.core.files.base import ContentFile
from django.core.mail.backends.base import BaseEmailBackend

from ... import settings as mailviewer_settings


class EmailBackend(BaseEmailBackend):
    """
    An email backend to use during testing and local development with Django Mail Viewer.

    Uses a Django model to store sent emails so that they can be easily retrieved in multi-process environments such as
    using Django Channels or when sending an email from a python shell or for longer term storage and lookup.
    """

    def __init__(self, *args, **kwargs):
        self._backend_model = apps.get_model(mailviewer_settings.MAILVIEWER_DATABASE_BACKEND_MODEL)
        super().__init__(*args, **kwargs)

    def _parse_email_attachment(self, message, decode_file=True):
        """
        Parse an attachment out of an email.message.Message object.

        params:
        msg: email.message.Message object
        decode_file: Boolean whether to decode the base64 encoded file to an actual file or not
        """
        # copied from views.SingleEmailMixin._parse_email_attachment()
        # TODO: deduplicate this. I think it maybe could live on BaseEmailBackend safely
        content_disposition = message.get("Content-Disposition", None)
        if content_disposition:
            dispositions = content_disposition.strip().split(";")
            if bool(content_disposition and dispositions[0].lower() == "attachment"):
                if decode_file:
                    file_data = message.get_payload(decode=True)
                    attachment = BytesIO(file_data)
                    attachment.size = len(file_data)

                    attachment.content_type = message.get_content_type()
                    attachment.name = None
                    attachment.create_date = None
                    attachment.mod_date = None
                    attachment.read_date = None
                else:
                    attachment = None
                for param in dispositions[1:]:
                    name, value = param.split("=")
                    name = name.lower()

                    # Since I am terrible and left terrible comments I am assuming the
                    # datetime TODO comments below mean to store as a datetime.datetime object
                    if name == "filename":
                        attachment.name = value
                    elif name == "create-date":
                        attachment.create_date = value  # TODO: datetime
                    elif name == "modification-date":
                        attachment.mod_date = value  # TODO: datetime
                    elif name == "read-date":
                        attachment.read_date = value  # TODO: datetime
                return {
                    'filename': Path(message.get_filename()).name,
                    'content_type': message.get_content_type(),
                    'file': attachment,
                }
        return None

    def send_messages(self, messages):
        msg_count = 0
        for m in messages:
            # Create db model instances
            message = m.message()
            if message.is_multipart():
                # TODO: Should this really be done recursively? I believe forwarded emails may
                # have multiple layers of parts/dispositions
                message_id = message.get('message-id')
                main_message = None
                for i, part in enumerate(message.walk()):
                    content_type = part.get_content_type()
                    charset = part.get_param('charset')
                    # handle attachments - probably need to look at SingleEmailMixin._parse_email_attachment()
                    # and make that more reusable
                    content_disposition = part.get("Content-Disposition", None)
                    if content_disposition:
                        # attachment_data = part.get_payload(decode=True)
                        attachment_data = self._parse_email_attachment(part)
                        file_attachment = ContentFile(
                            attachment_data.get('file').read(), name=attachment_data.get('filename', 'attachment')
                        )
                        content = ''
                    elif content_type in ['text/plain', 'text/html']:
                        content = part.get_payload(decode=True).decode(charset, errors='replace')
                        file_attachment = ''
                    else:
                        # the main multipart/alternative message for multipart messages has no content/payload
                        # TODO: handle file attachments
                        content = ''
                        file_attachment = ''
                    message_id = part.get('message-id', '')  # do sub-parts have a message-id?
                    p = self._backend_model(
                        message_id=message_id,
                        content=content,
                        file_attachment=file_attachment,
                        parent=main_message,
                        message_headers=json.dumps(dict(part.items())),
                    )
                    p.save()
                    if i == 0:
                        main_message = p
            else:
                message_id = message.get('message-id')
                main_message = self._backend_model(
                    message_id=message_id,
                    content=message.get_payload(),
                    message_headers=json.dumps(dict(message.items())),
                )
                main_message.save()

            msg_count += 1
        return msg_count

    def get_message(self, lookup_id):
        """
        Look up and return a specific message in the outbox
        """
        # Should this look at the db model and turn these into email.message.Message objects?
        # or should the views be updated so that more of their logic lives in the EmailBackend?
        # or should there be a layer in between or some sort of adapter pattern to make the db based email message
        # look/act like an email.message.Message? I lean towards just moving logic to the EmailBackend but may need
        # some combo of the two for the views/templates to work nicely.
        return self._backend_model.objects.filter(message_id=lookup_id, parent=None).first()

    def get_outbox(self, *args, **kwargs):
        """
        Get the outbox used by this backend.  This backend returns a copy of mail.outbox.
        May add pagination args/kwargs.
        """
        return self._backend_model.objects.filter(parent=None)

    def delete_message(self, message_id: str):
        """
        Remove the message with the given id from the mailbox
        """
        self._backend_model.objects.filter(message_id=message_id).delete()



# Generated by Django 3.1.2 on 2021-02-19 22:32

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='EmailMessage',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('message_id', models.CharField(blank=True, default='', max_length=250)),
                ('message_headers', models.TextField()),
                ('content', models.TextField(blank=True, default='')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('file_attachment', models.FileField(blank=True, default='', upload_to='mail_viewer_attachments')),
                ('parent', models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='parts', to='mail_viewer_database_backend.emailmessage')),
            ],
            options={
                'db_table': 'mail_viewer_emailmessage',
                'ordering': ('id',),
            },
        ),
        migrations.AddIndex(
            model_name='emailmessage',
            index=models.Index(fields=['message_id'], name='mail_viewer_message_f7eea6_idx'),
        ),
    ]
